@@ -110,14 +110,13 @@ import datetime
 import json
 import math
 import re
-import sys
 import uuid
 
 from pymongo.errors import ConfigurationError
 
 import bson
-from bson import EPOCH_AWARE, EPOCH_NAIVE, RE_TYPE, SON
-from bson.binary import (Binary, JAVA_LEGACY, CSHARP_LEGACY, OLD_UUID_SUBTYPE,
+from bson import EPOCH_AWARE, RE_TYPE, SON
+from bson.binary import (Binary, UuidRepresentation, ALL_UUID_SUBTYPES,
                          UUID_SUBTYPE)
 from bson.code import Code
 from bson.codec_options import CodecOptions
@@ -245,9 +244,9 @@ class JSONOptions(CodecOptions):
       - `document_class`: BSON documents returned by :func:`loads` will be
         decoded to an instance of this class. Must be a subclass of
         :class:`collections.MutableMapping`. Defaults to :class:`dict`.
-      - `uuid_representation`: The BSON representation to use when encoding
-        and decoding instances of :class:`uuid.UUID`. Defaults to
-        :const:`~bson.binary.PYTHON_LEGACY`.
+      - `uuid_representation`: The :class:`~bson.binary.UuidRepresentation`
+        to use when encoding and decoding instances of :class:`uuid.UUID`.
+        Defaults to :const:`~bson.binary.UuidRepresentation.PYTHON_LEGACY`.
       - `tz_aware`: If ``True``, MongoDB Extended JSON's *Strict mode* type
         `Date` will be decoded to timezone aware instances of
         :class:`datetime.datetime`. Otherwise they will be naive. Defaults
@@ -312,6 +311,26 @@ class JSONOptions(CodecOptions):
                     self.json_mode,
                     super(JSONOptions, self)._arguments_repr()))
 
+    def with_options(self, **kwargs):
+        """
+        Make a copy of this JSONOptions, overriding some options::
+
+            >>> from bson.json_util import CANONICAL_JSON_OPTIONS
+            >>> CANONICAL_JSON_OPTIONS.tz_aware
+            True
+            >>> json_options = CANONICAL_JSON_OPTIONS.with_options(tz_aware=False)
+            >>> json_options.tz_aware
+            False
+
+        .. versionadded:: 3.12
+        """
+        opts = self._asdict()
+        for opt in ('strict_number_long', 'datetime_representation',
+                    'strict_uuid', 'json_mode'):
+            opts[opt] = kwargs.get(opt, getattr(self, opt))
+        opts.update(kwargs)
+        return JSONOptions(**opts)
+
 
 LEGACY_JSON_OPTIONS = JSONOptions(json_mode=JSONMode.LEGACY)
 """:class:`JSONOptions` for encoding to PyMongo's legacy JSON format.
@@ -450,7 +469,7 @@ def object_hook(dct, json_options=DEFAULT_JSON_OPTIONS):
     if "$code" in dct:
         return _parse_canonical_code(dct)
     if "$uuid" in dct:
-        return _parse_legacy_uuid(dct)
+        return _parse_legacy_uuid(dct, json_options)
     if "$undefined" in dct:
         return None
     if "$numberLong" in dct:
@@ -485,23 +504,34 @@ def _parse_legacy_regex(doc):
     return Regex(pattern, flags)
 
 
-def _parse_legacy_uuid(doc):
+def _parse_legacy_uuid(doc, json_options):
     """Decode a JSON legacy $uuid to Python UUID."""
     if len(doc) != 1:
         raise TypeError('Bad $uuid, extra field(s): %s' % (doc,))
-    return uuid.UUID(doc["$uuid"])
+    if not isinstance(doc["$uuid"], text_type):
+        raise TypeError('$uuid must be a string: %s' % (doc,))
+    if json_options.uuid_representation == UuidRepresentation.UNSPECIFIED:
+        return Binary.from_uuid(uuid.UUID(doc["$uuid"]))
+    else:
+        return uuid.UUID(doc["$uuid"])
 
 
 def _binary_or_uuid(data, subtype, json_options):
     # special handling for UUID
-    if subtype == OLD_UUID_SUBTYPE:
-        if json_options.uuid_representation == CSHARP_LEGACY:
-            return uuid.UUID(bytes_le=data)
-        if json_options.uuid_representation == JAVA_LEGACY:
-            data = data[7::-1] + data[:7:-1]
-        return uuid.UUID(bytes=data)
-    if subtype == UUID_SUBTYPE:
-        return uuid.UUID(bytes=data)
+    if subtype in ALL_UUID_SUBTYPES:
+        uuid_representation = json_options.uuid_representation
+        binary_value = Binary(data, subtype)
+        if uuid_representation == UuidRepresentation.UNSPECIFIED:
+            return binary_value
+        if subtype == UUID_SUBTYPE:
+            # Legacy behavior: use STANDARD with binary subtype 4.
+            uuid_representation = UuidRepresentation.STANDARD
+        elif uuid_representation == UuidRepresentation.STANDARD:
+            # subtype == OLD_UUID_SUBTYPE
+            # Legacy behavior: STANDARD is the same as PYTHON_LEGACY.
+            uuid_representation = UuidRepresentation.PYTHON_LEGACY
+        return binary_value.as_uuid(uuid_representation)
+
     if PY3 and subtype == 0:
         return data
     return Binary(data, subtype)
@@ -795,15 +825,9 @@ def default(obj, json_options=DEFAULT_JSON_OPTIONS):
         return _encode_binary(obj, 0, json_options)
     if isinstance(obj, uuid.UUID):
         if json_options.strict_uuid:
-            data = obj.bytes
-            subtype = OLD_UUID_SUBTYPE
-            if json_options.uuid_representation == CSHARP_LEGACY:
-                data = obj.bytes_le
-            elif json_options.uuid_representation == JAVA_LEGACY:
-                data = data[7::-1] + data[:7:-1]
-            elif json_options.uuid_representation == UUID_SUBTYPE:
-                subtype = UUID_SUBTYPE
-            return _encode_binary(data, subtype, json_options)
+            binval = Binary.from_uuid(
+                obj, uuid_representation=json_options.uuid_representation)
+            return _encode_binary(binval, binval.subtype, json_options)
         else:
             return {"$uuid": obj.hex}
     if isinstance(obj, Decimal128):
